@@ -33,7 +33,6 @@
 #include "mt-timer.h"
 #include "mt-cursor-manager.h"
 #include "mt-cursor.h"
-#include "mt-main.h"
 #include "mt-listener.h"
 #include "mt-sig-handler.h"
 
@@ -50,18 +49,32 @@ typedef struct _MtCliArgs
     gdouble  ssc_time;
     gdouble  dwell_time;
     gchar   *mode;
-    gint     pos_x;
-    gint     pos_y;
+    gchar   *geometry;
     gint     threshold;
     gboolean ssc_enabled;
     gboolean dwell_enabled;
     gboolean shutdown;
     gboolean daemonize;
     gboolean ctw;
-    gboolean no_animation;
     gboolean login;
 } MtCliArgs;
 
+typedef struct _MtData
+{
+    MtService   *service;
+    MtTimer     *ssc_timer;
+    MtTimer     *dwell_timer;
+    gint         direction;
+    gint         pointer_x;
+    gint         pointer_y;
+
+    /* state flags */
+    guint        dwell_drag_started    : 1;
+    guint        dwell_gesture_started : 1;
+    guint        override_cursor       : 1;
+    guint        ssc_finished          : 1;
+} MtData;
+
 static void
 mt_main_generate_motion_event (GdkScreen *screen, gint x, gint y)
 {
@@ -126,13 +139,14 @@ mt_main_set_cursor (MtData *mt, GdkCursorType type)
 static void
 mt_main_do_dwell_click (MtData *mt)
 {
-    MtClickType click_type;
+    MtDwellClickType click_type;
     MtSettings *ms;
 
     ms = mt_settings_get_default ();
     click_type = mt_service_get_click_type (mt->service);
 
-    if (ms->dwell_mode == DWELL_MODE_GESTURE && !mt->dwell_drag_started)
+    if (ms->dwell_mode == G_DESKTOP_MOUSE_DWELL_MODE_GESTURE &&
+        !mt->dwell_drag_started)
     {
         mt_main_generate_motion_event (mt_common_get_screen (),
                                        mt->pointer_x,
@@ -141,14 +155,14 @@ mt_main_do_dwell_click (MtData *mt)
 
     switch (click_type)
     {
-        case DWELL_CLICK_TYPE_SINGLE:
+        case MT_DWELL_CLICK_TYPE_SINGLE:
             mt_main_generate_button_event (mt, 1, CLICK, 60);
             break;
-        case DWELL_CLICK_TYPE_DOUBLE:
+        case MT_DWELL_CLICK_TYPE_DOUBLE:
             mt_main_generate_button_event (mt, 1, DOUBLE_CLICK, 10);
-            mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_SINGLE);
+            mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
             break;
-        case DWELL_CLICK_TYPE_DRAG:
+        case MT_DWELL_CLICK_TYPE_DRAG:
             if (!mt->dwell_drag_started)
             {
                 mt_main_generate_button_event (mt, 1, PRESS, CurrentTime);
@@ -161,12 +175,12 @@ mt_main_do_dwell_click (MtData *mt)
                 mt_main_set_cursor (mt, GDK_LEFT_PTR);
                 mt->dwell_drag_started = FALSE;
 
-                mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_SINGLE);
+                mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
             }
             break;
-        case DWELL_CLICK_TYPE_RIGHT:
+        case MT_DWELL_CLICK_TYPE_RIGHT:
             mt_main_generate_button_event (mt, 3, CLICK, 10);
-            mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_SINGLE);
+            mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
             break;
         default:
             g_warning ("Unknown click-type.");
@@ -187,33 +201,37 @@ below_threshold (MtData *mt, gint x, gint y)
     return (dx * dx + dy * dy) < (ms->dwell_threshold * ms->dwell_threshold);
 }
 
-static gint
+static GDesktopMouseDwellDirection
 mt_main_get_direction (MtData *mt, gint x, gint y)
 {
     gint dx, dy;
 
     dx = ABS (mt->pointer_x - x);
     dy = ABS (mt->pointer_y - y);
+
     if (mt->pointer_x < x)
     {
         if (dx > dy)
-            return DIRECTION_LEFT;
+            return G_DESKTOP_MOUSE_DWELL_DIRECTION_LEFT;
     }
     else
     {
         if (dx > dy)
-            return DIRECTION_RIGHT;
+            return G_DESKTOP_MOUSE_DWELL_DIRECTION_RIGHT;
     }
-    return mt->pointer_y < y ? DIRECTION_UP : DIRECTION_DOWN;
+
+    return mt->pointer_y < y ? G_DESKTOP_MOUSE_DWELL_DIRECTION_UP :
+                               G_DESKTOP_MOUSE_DWELL_DIRECTION_DOWN;
 }
 
 static gboolean
 mt_main_analyze_gesture (MtData *mt)
 {
     MtSettings *ms;
-    gint direction, x, y;
+    GDesktopMouseDwellDirection direction;
+    gint x, y;
 
-    if (mt_service_get_click_type (mt->service) == DWELL_CLICK_TYPE_DRAG)
+    if (mt_service_get_click_type (mt->service) == MT_DWELL_CLICK_TYPE_DRAG)
         return TRUE;
 
     gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
@@ -226,19 +244,19 @@ mt_main_analyze_gesture (MtData *mt)
 
     if (direction == ms->dwell_gesture_single)
     {
-        mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_SINGLE);
+        mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
     }
     else if (direction == ms->dwell_gesture_double)
     {
-        mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_DOUBLE);
+        mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_DOUBLE);
     }
     else if (direction == ms->dwell_gesture_drag)
     {
-        mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_DRAG);
+        mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_DRAG);
     }
     else if (direction == ms->dwell_gesture_secondary)
     {
-        mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_RIGHT);
+        mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_RIGHT);
     }
     else
     {
@@ -293,7 +311,7 @@ dwell_timer_finished (MtTimer *timer, MtData *mt)
     ms = mt_settings_get_default ();
     mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
 
-    if (ms->dwell_mode == DWELL_MODE_CTW)
+    if (ms->dwell_mode == G_DESKTOP_MOUSE_DWELL_MODE_WINDOW)
     {
         mt_main_do_dwell_click (mt);
     }
@@ -349,7 +367,7 @@ mt_dwell_click_cancel (MtData *mt)
         mt->dwell_drag_started = FALSE;
     }
 
-    mt_service_set_click_type (mt->service, DWELL_CLICK_TYPE_SINGLE);
+    mt_service_set_click_type (mt->service, MT_DWELL_CLICK_TYPE_SINGLE);
 }
 
 static void
@@ -420,35 +438,31 @@ global_button_event (MtListener *listener,
 }
 
 static void
-cursor_overlay_time (MtData  *mt,
-                     guchar  *image,
-                     gint     width,
-                     gint     height,
-                     MtTimer *timer,
-                     gdouble  time)
+cursor_overlay_time (guchar *image,
+                     gint    width,
+                     gint    height,
+                     gdouble target,
+                     gdouble elapsed)
 {
-    GdkColor c;
+    GdkColor color;
     GtkStyle *style;
     cairo_surface_t *surface;
     cairo_t *cr;
-    gdouble target;
+
+    style = gtk_widget_get_style (mt_ctw_get_window ());
+    color = style->bg[GTK_STATE_SELECTED];
 
     surface = cairo_image_surface_create_for_data (image,
                                                    CAIRO_FORMAT_ARGB32,
                                                    width, height,
                                                    width * 4);
     cr = cairo_create (surface);
-
-    style = gtk_widget_get_style (mt_ctw_get_window ());
-    c = style->bg[GTK_STATE_SELECTED];
-    target = mt_timer_get_target (timer);
-
     cairo_set_operator (cr, CAIRO_OPERATOR_ATOP);
-    cairo_rectangle (cr, 0, 0, width, height / (target / time));
+    cairo_rectangle (cr, 0, 0, width, (height * elapsed) / target);
     cairo_set_source_rgba (cr,
-                           c.red   / 65535.,
-                           c.green / 65535.,
-                           c.blue  / 65535.,
+                           color.red   / 65535.,
+                           color.green / 65535.,
+                           color.blue  / 65535.,
                            0.60);
     cairo_fill (cr);
     cairo_destroy (cr);
@@ -456,62 +470,54 @@ cursor_overlay_time (MtData  *mt,
 }
 
 static void
-mt_main_update_cursor (MtData  *mt,
-                       MtTimer *timer,
-                       gdouble  time)
+mt_main_timer_tick (MtTimer *timer, gdouble elapsed, gpointer data)
 {
-    MtCursor *new_cursor;
-    const gchar *name;
-    gushort xhot, yhot;
-    guchar *image;
-    gushort width, height;
+    MtCursorManager *manager;
+    MtCursor *current_cursor, *new_cursor;
 
-    image = mt_cursor_get_image_copy (mt->cursor);
-    if (image)
+    manager = mt_cursor_manager_get_default ();
+    current_cursor = mt_cursor_manager_get_current_cursor (manager);
+
+    if (current_cursor)
     {
-        mt_cursor_get_dimension (mt->cursor, &width, &height);
-        cursor_overlay_time (mt, image, width, height, timer, time);
+        const gchar *name;
+        gushort width, height, xhot, yhot;
+        gdouble target;
+        guchar *image;
 
-        name = mt_cursor_get_name (mt->cursor);
-        mt_cursor_get_hotspot (mt->cursor, &xhot, &yhot);
+        /* get cursor info */
+        name = mt_cursor_get_name (current_cursor);
+        image = mt_cursor_get_image_copy (current_cursor);
+        mt_cursor_get_dimension (current_cursor, &width, &height);
+        mt_cursor_get_hotspot (current_cursor, &xhot, &yhot);
+
+        g_object_unref (current_cursor);
+
+        target = mt_timer_get_target (timer);
+
+        /* paint overlay */
+        cursor_overlay_time (image, width, height, target, elapsed);
+
+        /* create and set new cursor */
         new_cursor = mt_cursor_new (name, image, width, height, xhot, yhot);
-        mt_cursor_manager_set_cursor (mt_cursor_manager_get_default (), new_cursor);
-
-        g_object_unref (new_cursor);
-        g_free (image);
+        if (new_cursor)
+        {
+            mt_cursor_manager_set_cursor (manager, new_cursor);
+            g_object_unref (new_cursor);
+        }
     }
 }
 
 static void
-mt_main_timer_tick (MtTimer *timer,
-                    gdouble  time,
-                    MtData  *mt)
-{
-    MtSettings *ms;
-
-    ms = mt_settings_get_default ();
-    if (ms->animate_cursor && mt->cursor)
-    {
-        mt_main_update_cursor (mt, timer, time);
-    }
-}
-
-static void
-cursor_cache_cleared (MtCursorManager *manager, MtData *mt)
-{
-    mt->cursor = mt_cursor_manager_current_cursor (manager);
-}
-
-static void
-cursor_changed (MtCursorManager *manager,
-                const gchar     *name,
-                MtData          *mt)
+mt_main_cursor_changed (MtCursorManager *manager,
+                        const gchar     *name,
+                        MtData          *mt)
 {
     if (!mt->dwell_gesture_started)
     {
+        /* Remove me, I'm weird */
         mt->override_cursor = !g_str_equal (name, "left_ptr");
     }
-    mt->cursor = mt_cursor_manager_lookup_cursor (manager, name);
 }
 
 static void
@@ -525,6 +531,7 @@ mt_main_sig_handler (MtSigHandler *sigh,
                      gint          signal_id,
                      gpointer      data)
 {
+    mt_ctw_save_geometry ();
     signal_handler (signal_id);
 }
 
@@ -582,9 +589,9 @@ mt_parse_options (int *argc, char ***argv)
     GOptionEntry entries[] =
     {
         {"dwell", 0, 0, G_OPTION_ARG_NONE, &ca.dwell_enabled,
-            N_("Enable dwell click"), 0},
+            N_("Enable dwell click"), NULL},
         {"ssc", 0, 0, G_OPTION_ARG_NONE, &ca.ssc_enabled,
-            N_("Enable simulated secondary click"), 0},
+            N_("Enable simulated secondary click"), NULL},
         {"dwell-time", 0, 0, G_OPTION_ARG_DOUBLE, &ca.dwell_time,
             N_("Time to wait before a dwell click"), "[0.2-3.0]"},
         {"ssc-time", 0, 0, G_OPTION_ARG_DOUBLE, &ca.ssc_time,
@@ -592,21 +599,17 @@ mt_parse_options (int *argc, char ***argv)
         {"dwell-mode", 'm', 0, G_OPTION_ARG_STRING, &ca.mode,
             N_("Set the active dwell mode"), "[window|gesture]"},
         {"show-ctw", 'c', 0, G_OPTION_ARG_NONE, &ca.ctw,
-            N_("Show a click-type window"), 0},
-        {"ctw-x", 'x', 0, G_OPTION_ARG_INT, &ca.pos_x,
-            N_("Click-type window X position"), 0},
-        {"ctw-y", 'y', 0, G_OPTION_ARG_INT, &ca.pos_y,
-            N_("Click-type window Y position"), 0},
+            N_("Show a click-type window"), NULL},
         {"threshold", 't', 0, G_OPTION_ARG_INT, &ca.threshold,
             N_("Ignore small pointer movements"), "[0-30]"},
+        {"geometry", 'g', 0, G_OPTION_ARG_STRING, &ca.geometry,
+            N_("Click-type window geometry"), "WIDTHxHEIGHT+X+Y"},
         {"shutdown", 's', 0, G_OPTION_ARG_NONE, &ca.shutdown,
-            N_("Shut down mousetweaks"), 0},
-        {"disable-animation", 0, 0, G_OPTION_ARG_NONE, &ca.no_animation,
-            N_("Disable cursor animations"), 0},
+            N_("Shut down mousetweaks"), NULL},
         {"daemonize", 0, 0, G_OPTION_ARG_NONE, &ca.daemonize,
-            N_("Start mousetweaks as a daemon"), 0},
+            N_("Start mousetweaks as a daemon"), NULL},
         {"login", 0, 0, G_OPTION_ARG_NONE, &ca.login,
-            N_("Start mousetweaks in login mode"), 0},
+            N_("Start mousetweaks in login mode"), NULL},
         { NULL }
     };
 
@@ -614,15 +617,13 @@ mt_parse_options (int *argc, char ***argv)
     ca.ssc_time      = -1.;
     ca.dwell_time    = -1.;
     ca.mode          = NULL;
-    ca.pos_x         = -1;
-    ca.pos_y         = -1;
+    ca.geometry      = NULL;
     ca.threshold     = -1;
     ca.ssc_enabled   = FALSE;
     ca.dwell_enabled = FALSE;
     ca.shutdown      = FALSE;
     ca.daemonize     = FALSE;
     ca.ctw           = FALSE;
-    ca.no_animation  = FALSE;
     ca.login         = FALSE;
 
     /* parse */
@@ -680,11 +681,11 @@ mt_main (int argc, char **argv, MtCliArgs cli_args)
     ms = mt_settings_get_default ();
 
     /* bind timers */
-    g_settings_bind (ms->mt_settings, KEY_SSC_TIME,
+    g_settings_bind (ms->a11y_settings, KEY_SSC_TIME,
                      mt->ssc_timer, "target-time",
                      G_SETTINGS_BIND_DEFAULT | G_SETTINGS_BIND_NO_SENSITIVITY);
 
-    g_settings_bind (ms->mt_settings, KEY_DWELL_TIME,
+    g_settings_bind (ms->a11y_settings, KEY_DWELL_TIME,
                      mt->dwell_timer, "target-time",
                      G_SETTINGS_BIND_DEFAULT | G_SETTINGS_BIND_NO_SENSITIVITY);
 
@@ -701,29 +702,29 @@ mt_main (int argc, char **argv, MtCliArgs cli_args)
         ms->dwell_threshold = cli_args.threshold;
     if (cli_args.ctw)
         ms->ctw_visible = cli_args.ctw;
-    if (cli_args.no_animation)
-        ms->animate_cursor = !cli_args.no_animation;
     if (cli_args.mode)
     {
         if (g_str_equal (cli_args.mode, "gesture"))
-            ms->dwell_mode = DWELL_MODE_GESTURE;
+            ms->dwell_mode = G_DESKTOP_MOUSE_DWELL_MODE_GESTURE;
         else if (g_str_equal (cli_args.mode, "window"))
-            ms->dwell_mode = DWELL_MODE_CTW;
+            ms->dwell_mode = G_DESKTOP_MOUSE_DWELL_MODE_WINDOW;
 
         g_free (cli_args.mode);
     }
+    if (cli_args.geometry)
+    {
+        g_free (ms->ctw_geometry);
+        ms->ctw_geometry = cli_args.geometry;
+    }
 
     /* init click-type window */
-    if (!mt_ctw_init (cli_args.pos_x, cli_args.pos_y))
+    if (!mt_ctw_init ())
         goto CLEANUP;
 
     /* init cursor animation */
     manager = mt_cursor_manager_get_default ();
-    mt->cursor = mt_cursor_manager_current_cursor (manager);
     g_signal_connect (manager, "cursor_changed",
-                      G_CALLBACK (cursor_changed), mt);
-    g_signal_connect (manager, "cache_cleared",
-                      G_CALLBACK (cursor_cache_cleared), mt);
+                      G_CALLBACK (mt_main_cursor_changed), mt);
 
     /* init mouse listener */
     listener = mt_listener_get_default ();
